@@ -1,5 +1,5 @@
 /*  smplayer, GUI front-end for mplayer.
-    Copyright (C) 2006-2016 Ricardo Villalba <rvm@users.sourceforge.net>
+    Copyright (C) 2006-2017 Ricardo Villalba <rvm@users.sourceforge.net>
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #include <QApplication>
 #include <QMimeData>
 #include <QDomDocument>
+#include <QDesktopServices>
 #include <QDebug>
 
 #if QT_VERSION >= 0x050000
@@ -61,6 +62,10 @@
 #include "extensions.h"
 #include "guiconfig.h"
 
+#ifdef CHROMECAST_SUPPORT
+#include "chromecast.h"
+#endif
+
 #ifdef PLAYLIST_DOWNLOAD
 #include "inputurl.h"
 #include "youtube/loadpage.h"
@@ -223,6 +228,21 @@ QList<QStandardItem *> PLItem::items() {
 	return l;
 }
 
+void PLItem::setExtraParams(const QStringList & pars) {
+	setData(pars, Role_Params);
+}
+
+QStringList PLItem::extraParams() {
+	return data(Role_Params).toStringList();
+}
+
+void PLItem::setVideoURL(const QString & url) {
+	setData(url, Role_Video_URL);
+}
+
+QString PLItem::videoURL() {
+	return data(Role_Video_URL).toString();
+}
 
 /* ----------------------------------------------------------- */
 
@@ -241,6 +261,9 @@ Playlist::Playlist(QWidget * parent, Qt::WindowFlags f)
 	, ignore_player_errors(false)
 	, change_name(true)
 	, save_dirs(true)
+#ifdef PLAYLIST_DELETE_FROM_DISK
+	, allow_delete_from_disk(false)
+#endif
 {
 	playlist_path = "";
 	latest_dir = "";
@@ -299,7 +322,7 @@ Playlist::Playlist(QWidget * parent, Qt::WindowFlags f)
 	connect(downloader, SIGNAL(errorOcurred(int, QString)), this, SLOT(errorOcurred(int, QString)));
 
 	history_urls = new URLHistory;
-	history_urls->addUrl("http://smplayer.info/onlinetv.php");
+	history_urls->addUrl("http://smplayer.info/sample.m3u8");
 #endif
 }
 
@@ -485,8 +508,24 @@ void Playlist::createActions() {
 	editAct = new MyAction(this, "pl_edit", false);
 	connect( editAct, SIGNAL(triggered()), this, SLOT(editCurrentItem()) );
 
+#ifdef PLAYLIST_DELETE_FROM_DISK
 	deleteSelectedFileFromDiskAct = new MyAction(this, "pl_delete_from_disk");
 	connect( deleteSelectedFileFromDiskAct, SIGNAL(triggered()), this, SLOT(deleteSelectedFileFromDisk()));
+#endif
+
+	copyURLAct = new MyAction(this, "pl_copy_url");
+	connect( copyURLAct, SIGNAL(triggered()), this, SLOT(copyURL()));
+
+	openFolderAct = new MyAction(this, "pl_open_folder");
+	connect( openFolderAct, SIGNAL(triggered()), this, SLOT(openFolder()));
+
+#ifdef CHROMECAST_SUPPORT
+	playOnChromecastAct = new MyAction(this, "pl_chromecast");
+	connect( playOnChromecastAct, SIGNAL(triggered()), this, SLOT(playOnChromecast()));
+#else
+	openURLInWebAct = new MyAction(this, "pl_url_in_web");
+	connect( openURLInWebAct, SIGNAL(triggered()), this, SLOT(openURLInWeb()));
+#endif
 
 	showSearchAct = new MyAction(this, "pl_show_search", false);
 	showSearchAct->setCheckable(true);
@@ -609,7 +648,16 @@ void Playlist::createToolbar() {
 	popup->addAction(playAct);
 	popup->addAction(removeSelectedAct);
 	popup->addAction(editAct);
+#ifdef PLAYLIST_DELETE_FROM_DISK
 	popup->addAction(deleteSelectedFileFromDiskAct);
+#endif
+	popup->addAction(copyURLAct);
+	popup->addAction(openFolderAct);
+#ifdef CHROMECAST_SUPPORT
+	popup->addAction(playOnChromecastAct);
+#else
+	popup->addAction(openURLInWebAct);
+#endif
 	popup->addSeparator();
 	popup->addAction(showPositionColumnAct);
 	popup->addAction(showNameColumnAct);
@@ -653,10 +701,21 @@ void Playlist::retranslateStrings() {
 	addUrlsAct->change( tr("Add &URL(s)") );
 
 	// Remove actions
-	removeSelectedAct->change( tr("Remove &selected") );
+	removeSelectedAct->change( Images::icon("delete"), tr("Remove &selected") );
 	removeAllAct->change( tr("Remove &all") );
 
+#ifdef PLAYLIST_DELETE_FROM_DISK
 	deleteSelectedFileFromDiskAct->change( tr("&Delete file from disk") );
+#endif
+
+	copyURLAct->change( Images::icon("copy"), tr("&Copy file path to clipboard") );
+	openFolderAct->change( Images::icon("openfolder"), tr("&Open source folder") );
+
+#ifdef CHROMECAST_SUPPORT
+	playOnChromecastAct->change( Images::icon("chromecast"), tr("Play on Chromec&ast") );
+#else
+	openURLInWebAct->change( tr("Open stream in &a web browser") );
+#endif
 
 	showSearchAct->change(Images::icon("find"), tr("Search"));
 
@@ -801,7 +860,7 @@ void Playlist::changeItem(int row, const QString & filename, const QString name,
 }
 */
 
-void Playlist::addItem(QString filename, QString name, double duration) {
+void Playlist::addItem(QString filename, QString name, double duration, QStringList params, QString video_url) {
 	//qDebug() << "Playlist::addItem:" << filename;
 
 	#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
@@ -821,6 +880,8 @@ void Playlist::addItem(QString filename, QString name, double duration) {
 		}
 
 	PLItem * i = new PLItem(filename, name, duration);
+	i->setExtraParams(params);
+	i->setVideoURL(video_url);
 	i->setPosition(count()+1);
 	table->appendRow(i->items());
 
@@ -890,6 +951,7 @@ void Playlist::load_m3u(QString file, M3UFormat format) {
 		QString filename="";
 		QString name="";
 		double duration=0;
+		QStringList extra_params;
 
 		QTextStream stream( &f );
 
@@ -923,6 +985,12 @@ void Playlist::load_m3u(QString file, M3UFormat format) {
 				if (fields.count() >= 2) name = fields[1];
 			}
 			else
+			if (line.startsWith("#EXTVLCOPT:")) {
+				QString par = line.mid(11);
+				qDebug() << "Playlist::load_m3u: EXTVLCOPT:" << par;
+				extra_params << par;
+			}
+			else
 			if (line.startsWith("#")) {
 				// Comment
 				// Ignore
@@ -938,9 +1006,11 @@ void Playlist::load_m3u(QString file, M3UFormat format) {
 					}
 				}
 				name.replace("&#44;", ",");
-				addItem( filename, name, duration );
+				//qDebug() << "Playlist::load_m3u: extra_params:" << extra_params;
+				addItem( filename, name, duration, extra_params );
 				name=""; 
 				duration = 0;
+				extra_params.clear();
 			}
 		}
 		f.close();
@@ -1085,6 +1155,13 @@ bool Playlist::save_m3u(QString file) {
 			stream << "#EXTINF:";
 			stream << i->duration() << ",";
 			stream << name << "\n";
+
+			// Save extra params
+			QStringList params = i->extraParams();
+			foreach(QString par, params) {
+				stream << "#EXTVLCOPT:" << par << "\n";
+			}
+
 			// Try to save the filename as relative instead of absolute
 			if (filename.startsWith( dir_path )) {
 				filename = filename.mid( dir_path.length() );
@@ -1333,6 +1410,56 @@ void Playlist::itemActivated(const QModelIndex & index ) {
 void Playlist::showPopup(const QPoint & pos) {
 	qDebug("Playlist::showPopup: x: %d y: %d", pos.x(), pos.y() );
 
+	QModelIndex index = listView->currentIndex();
+	if (!index.isValid()) {
+		playAct->setEnabled(false);
+		removeSelectedAct->setEnabled(false);
+		editAct->setEnabled(false);
+		#ifdef PLAYLIST_DELETE_FROM_DISK
+		deleteSelectedFileFromDiskAct->setEnabled(false);
+		#endif
+		copyURLAct->setEnabled(false);
+		openFolderAct->setEnabled(false);
+		#ifdef CHROMECAST_SUPPORT
+		playOnChromecastAct->setEnabled(false);
+		#else
+		openURLInWebAct->setEnabled(false);
+		#endif
+	} else {
+		playAct->setEnabled(true);
+		removeSelectedAct->setEnabled(true);
+		editAct->setEnabled(true);
+		#ifdef PLAYLIST_DELETE_FROM_DISK
+		deleteSelectedFileFromDiskAct->setEnabled(allow_delete_from_disk);
+		#endif
+		copyURLAct->setEnabled(true);
+		openFolderAct->setEnabled(true);
+		#ifdef CHROMECAST_SUPPORT
+		playOnChromecastAct->setEnabled(true);
+		#else
+		openURLInWebAct->setEnabled(true);
+		#endif
+
+		QModelIndex s_index = proxy->mapToSource(index);
+		int current = s_index.row();
+		PLItem * i = itemData(current);
+		QString filename = i->filename();
+		QFileInfo fi(filename);
+
+		if (fi.exists()) {
+			copyURLAct->setText( tr("&Copy file path to clipboard") );
+			#ifndef CHROMECAST_SUPPORT
+			openURLInWebAct->setEnabled(false);
+			#endif
+		} else {
+			copyURLAct->setText( tr("&Copy URL to clipboard") );
+			openFolderAct->setEnabled(false);
+			#ifdef PLAYLIST_DELETE_FROM_DISK
+			deleteSelectedFileFromDiskAct->setEnabled(false);
+			#endif
+		}
+	}
+
 	if (!popup->isVisible()) {
 		popup->move( listView->viewport()->mapToGlobal(pos) );
 		popup->show();
@@ -1356,13 +1483,21 @@ void Playlist::playItem( int n ) {
 		return;
 	}
 
-	QString filename = itemFromProxy(n)->filename();
+	PLItem * i = itemFromProxy(n);
+	QString filename = i->filename();
+	QStringList params = i->extraParams();
+
 	if (!filename.isEmpty()) {
 		setCurrentItem(n);
-		if (play_files_from_start) {
-			emit requestToPlayFile(filename, 0);
+
+		if (!params.isEmpty()) {
+			emit requestToPlayStream(filename, params);
 		} else {
-			emit requestToPlayFile(filename);
+			if (play_files_from_start) {
+				emit requestToPlayFile(filename, 0);
+			} else {
+				emit requestToPlayFile(filename);
+			}
 		}
 	}
 }
@@ -1430,6 +1565,7 @@ void Playlist::getMediaInfo(const MediaData & mdat) {
 	QString filename = mdat.filename;
 	double duration = mdat.duration;
 	QString artist = mdat.clip_artist;
+	QString video_url = mdat.stream_path;
 
 	#if defined(Q_OS_WIN) || defined(Q_OS_OS2)
 	filename = Helper::changeSlashes(filename);
@@ -1469,6 +1605,7 @@ void Playlist::getMediaInfo(const MediaData & mdat) {
 			if (i->duration() == 1) {
 				i->setDuration(duration);
 			}
+			i->setVideoURL(video_url);
 		}
 	}
 }
@@ -1762,9 +1899,12 @@ void Playlist::editItem(int row) {
 	}
 }
 
+#ifdef PLAYLIST_DELETE_FROM_DISK
 void Playlist::deleteSelectedFileFromDisk() {
 	qDebug("Playlist::deleteSelectedFileFromDisk");
 
+	if (!allow_delete_from_disk) return;
+
 	QModelIndex index = listView->currentIndex();
 	if (!index.isValid()) return;
 
@@ -1812,6 +1952,90 @@ void Playlist::deleteSelectedFileFromDisk() {
 			tr("It's not possible to delete '%1' from the filesystem.").arg(filename));
 	}
 }
+#endif
+
+void Playlist::copyURL() {
+	qDebug("Playlist::copyURL");
+
+	QModelIndexList indexes = listView->selectionModel()->selectedRows();
+	int count = indexes.count();
+
+	QString text;
+
+	for (int n = 0; n < count; n++) {
+		QModelIndex s_index = proxy->mapToSource(indexes.at(n));
+		int current = s_index.row();
+		text += itemData(current)->filename();
+		if (n < count-1) {
+			#ifdef Q_OS_WIN
+			text += "\r\n";
+			#else
+			text += "\n";
+			#endif
+		}
+	}
+
+	if (!text.isEmpty()) QApplication::clipboard()->setText(text);
+}
+
+void Playlist::openFolder() {
+	qDebug("Playlist::openFolder");
+
+	QModelIndex index = listView->currentIndex();
+	if (!index.isValid()) return;
+	QModelIndex s_index = proxy->mapToSource(index);
+	int current = s_index.row();
+	PLItem * i = itemData(current);
+	QString filename = i->filename();
+
+	qDebug() << "Playlist::openFolder: filename:" << filename;
+
+	QFileInfo fi(filename);
+	if (fi.exists()) {
+		QString src_folder = fi.absolutePath();
+		QDesktopServices::openUrl(QUrl::fromLocalFile(src_folder));
+	}
+}
+
+#ifdef CHROMECAST_SUPPORT
+void Playlist::playOnChromecast() {
+	qDebug("Playlist::playOnChromecast");
+
+	QModelIndex index = listView->currentIndex();
+	if (!index.isValid()) return;
+	QModelIndex s_index = proxy->mapToSource(index);
+	int current = s_index.row();
+	PLItem * i = itemData(current);
+	QString filename = i->filename();
+	QString video_url = i->videoURL();
+
+	QString url = filename;
+	if (!video_url.isEmpty()) url = video_url;
+
+	if (QFile::exists(filename)) {
+		Chromecast::instance()->openLocal(url, i->name());
+	} else {
+		Chromecast::instance()->openStream(url, i->name());
+	}
+}
+#else
+void Playlist::openURLInWeb() {
+	qDebug("Playlist::openURLInWeb");
+
+	QModelIndex index = listView->currentIndex();
+	if (!index.isValid()) return;
+	QModelIndex s_index = proxy->mapToSource(index);
+	int current = s_index.row();
+	PLItem * i = itemData(current);
+	QString filename = i->filename();
+	QString video_url = i->videoURL();
+
+	QString url = filename;
+	if (!video_url.isEmpty()) url = video_url;
+
+	QDesktopServices::openUrl(QUrl(url));
+}
+#endif
 
 // Drag&drop
 void Playlist::dragEnterEvent( QDragEnterEvent *e ) {
@@ -1937,6 +2161,10 @@ void Playlist::saveSettings() {
 	set->setValue( "size", size() );
 #endif
 
+#ifdef PLAYLIST_DELETE_FROM_DISK
+	set->setValue("allow_delete_from_disk", allow_delete_from_disk);
+#endif
+
 	set->setValue(QString("header_state/2/%1").arg(Helper::qtVersion()), listView->horizontalHeader()->saveState());
 
 	set->setValue( "sort_column", proxy->sortColumn() );
@@ -1966,6 +2194,8 @@ void Playlist::saveSettings() {
 			set->setValue( QString("item_%1_filename").arg(n), i->filename() );
 			set->setValue( QString("item_%1_duration").arg(n), i->duration() );
 			set->setValue( QString("item_%1_name").arg(n), i->name() );
+			set->setValue( QString("item_%1_params").arg(n), i->extraParams() );
+			set->setValue( QString("item_%1_video_url").arg(n), i->videoURL() );
 		}
 		set->endArray();
 		set->setValue( "current_item", findCurrentItem() );
@@ -2012,6 +2242,10 @@ void Playlist::loadSettings() {
 	resize( set->value("size", size()).toSize() );
 #endif
 
+#ifdef PLAYLIST_DELETE_FROM_DISK
+	allow_delete_from_disk = set->value("allow_delete_from_disk", allow_delete_from_disk).toBool();
+#endif
+
 	listView->horizontalHeader()->restoreState(set->value(QString("header_state/2/%1").arg(Helper::qtVersion()), QByteArray()).toByteArray());
 
 	int sort_column = set->value("sort_column", COL_NUM).toInt();
@@ -2044,7 +2278,9 @@ void Playlist::loadSettings() {
 			filename = set->value( QString("item_%1_filename").arg(n), "" ).toString();
 			duration = set->value( QString("item_%1_duration").arg(n), -1 ).toDouble();
 			name = set->value( QString("item_%1_name").arg(n), "" ).toString();
-			addItem( filename, name, duration );
+			QStringList params = set->value( QString("item_%1_params").arg(n), QStringList()).toStringList();
+			QString video_url = set->value( QString("item_%1_video_url").arg(n), "").toString();
+			addItem( filename, name, duration, params, video_url );
 		}
 		set->endArray();
 		setCurrentItem( set->value( "current_item", -1 ).toInt() );
